<?php
/*
Plugin Name: CJ API for Wordpress
Plugin URI: http://www.agilitywebsolutions.com/wordpress/commission-junction-api-for-wordpress-plugin
Description: Allows users to add quickly add products from commission junction API to WordPress posts and pages.
Version: 0.1
Author: David Wilson
Author URI: http://www.agilitywebsolutions.com
License: GPL v2, see included "license.txt" file
*/

class CJ_API_Wordpress {

	//Install default data
	public function activate() {
		$options = array( 'advertiser_relationship' => 'joined' ,
		'result_row_template' => '<a href="%buy-url%"><img src="%image-url%" /></a><br />
<a href="%buy-url%">%name%</a>', 
		'result_row_css' =>'table.cj-results-table {
    border: 1px solid;
    color: #333333;  
}

table.cj-results-table tr.even {
    border: 1px solid;
    background: #FFF;  
}

table.cj-results-table tr.odd{
    border: 1px solid;
    background: #E9F1C2;  
}', 'cache_duration' => '3600');
    if ( ! get_option('cjapi_settings')){
      add_option('cjapi_settings' , $options);
    } else {
      update_option('cjapi_settings' , $options);
	}
	}
	
	//uninstall plugin
	public function uninstall() {
		delete_option('cjapi_settings');
	}

	//Add needed items on admin page load.
	public function add_admin_items() {
		add_options_page('CJ API for Wordpress Configuration', 'CJ API Options', 'administrator', 'cjapioptions', array($this,'cjapi_options'));
		add_meta_box('wp-cj-api', 'CJ API for Wordpress', array($this, 'showMetaBox'), 'post');
		add_meta_box('wp-cj-api', 'CJ API for Wordpress', array($this, 'showMetaBox'), 'page');
		
		//Check current page to add needed JavaScript
		global $pagenow;	
		if (false !== strpos($pagenow, 'post') || false !== strpos($pagenow, 'page')) {
			wp_enqueue_script('cjapi', plugins_url('js/wp-cj-api.js', __FILE__), array('jquery'));
			wp_enqueue_script('cjapi-jquery-ui', plugins_url('js/jquery-ui-1.7.2.custom.min.js', __FILE__));
		}
	}

	public function register_aioasettings() {
		register_setting( 'cjapi_settings', 'cjapi_settings' );
	}	

	public function cjapi_options() {
		include('includes/options-page.php');
	}
		
	public function showMetaBox() {
		include ('includes/search-box.php');
	}
		
	/*list results of CJ API search for shortcode*/
	public function cj_product_list($atts) {
		$options = get_option('cjapi_settings');	
		$duration = $options['cache_duration'];
		$params = shortcode_atts(array(
			'advertiser-ids' => $options['advertiser_relationship'],
			'serviceable-area' => $options['cj_region_code'],
			'keywords' => '',
			'lowprice' => '',
			'highprice' => '',
			'maxresults' => '10',
			'sortby' => '',
			'sortorder' => '',
			'isbn' => '',
			'upc' => '',
			'manufacturer-name' => '',
			'manufacturersku' => '',
			'advertisersku' => '',
			'lowsaleprice' => '',
			'highsaleprice' => '',
			'currency' => $options['cj_currency'],
			), $atts);
		//unique hash for these options	
		$hash = md5(implode($params));
		if (false === ($results = get_transient($hash))) {
			$xml = $this->get_xml_search_results($params);
			$template = $options['result_row_template'];
			
			if ( !is_wp_error($xml)) {
				$prod_list = '<table class="cj-results-table">';
				$div_class = 'odd';
				foreach($xml->products[0]->product as $item) {
					$this_row = $template;
					$replace_array = array(
						'%ad-id%' => $item->{'ad-id'},
						'%advertiser-id%' => $item->{'advertiser-id'},
						'%advertiser-name%' => $item->{'advertiser-name'},
						'%buy-url%' => $item->{'buy-url'},
						'%catalog-id%' => $item->{'catalog-id'},
						'%currency%' => $item->currency,
						'%description%' => $item->description,
						'%image-url%' => $item->{'image-url'},
						'%in-stock%' => $item->{'in-stock'},
						'%isbn%' => $item->isbn,
						'%manufacturer-name%' => $item->{'manufacturer-name'},
						'%manufacturer-sku%' => $item->{'manufacturer-sku'},
						'%name%' => $item->name,
						'%price%' => $item->{'price'},
						'%retail-price%' => $item->{'retail-price'},
						'%sale-price%' => $item->{'sale-price'},
						'%sku%' => $item->sku,
						'%upc%' => $item->upc,
						);
					foreach($replace_array as $search => $replace) {
                  if($search != "" && ($replace != "" || $replace != null)) {
                     $this_row = str_replace($search, $replace, $this_row);
                     }
					}
					$prod_list = $prod_list.'<tr class="'.$div_class.'"><td>'.$this_row.'</tr></td>';		
					$div_class = (strcmp($div_class, 'even') == 0) ? 'odd' : 'even';
				}
				$prod_list = $prod_list.'</table>';
			} else {
				$prod_list = $xml->get_error_message();
			}
			set_transient($hash, $prod_list, $duration);
			return $prod_list;
		} else {
			return $results;
		}
		}
	
	/* Return search results from AJAX request. */
	public function ajax_search() {
	$data = $_GET;
	$keywords = $data['keywords'];
   $ad_ids = is_null($data['advids']) ? $options['advertiser_relationship'] : $data['advids'];
	$params = array(
			'advertiser-ids' => $ad_ids,
			'serviceable-area' => $options['cj_region_code'],
			'keywords' => $keywords,
			'lowprice' => '',
			'highprice' => '',
			'maxresults' => '',
			'sortby' => '',
			'sortorder' => '',
			'isbn' => '',
			'upc' => '',
			'manufacturername' => '',
			'manufacturersku' => '',
			'advertisersku' => '',
			'lowsaleprice' => '',
			'highsaleprice' => '',
			'currency' => $options['cj_currency'],
			);
	
	$products = array();
	$xml = $this->get_xml_search_results($params);
	if ( !is_wp_error($xml)) {
		$products['products'] = $xml->products[0];
		echo json_encode($products);
		exit(); }
	else {
		echo $xml->get_error_message();
		}
	}
	
	//add custom style.
	public function add_css() {
		$options = get_option('cjapi_settings');	
		$style = $options['result_row_css'];
		
		echo '<style type="text/css">'.$style.'</style>';
	}
	
	function add_tinymce_button() {
		// Don't bother doing this stuff if the current user lacks permissions
		if ( ! current_user_can('edit_posts') && ! current_user_can('edit_pages') )
		  return;
 
		// Add only in Rich Editor mode
		if ( get_user_option('rich_editing') == 'true') {
		  add_filter("mce_external_plugins", array( $this, "add_cjapi_tinymce_plugin"));
		  add_filter('mce_buttons', array( $this, 'register_cjapi_button' ));
			}
	}
	
	function register_cjapi_button($buttons) {
		array_push($buttons, "|", "cjApiButton");
		return $buttons;
	}
 
	// Load the TinyMCE plugin : editor_plugin.js (wp2.5)
	function add_cjapi_tinymce_plugin($plugin_array) {
		$plugin_array['cjapi'] = plugins_url('js/editor_plugin.js', __FILE__);
		return $plugin_array;
	}
	
	
	private function get_xml_search_results($params) {
		//Get local settings
		$options = get_option('cjapi_settings');
		$site_id = $options['cj_site_id'];
		$api_key = $options['cj_api_key'];
		$region = $params['serviceable-area'];
		$joined = $params['advertiser-ids'];
		$keywords = urlencode($params['keywords']);
		$url = "https://product-search.api.cj.com/v2/product-search?website-id=$site_id&keywords=$keywords&serviceable-area=$region";
		//optional parameters, only add parameter if it is defined
		$url = $url."&advertiser-ids=$joined";
		$url = (strcmp($params['lowprice'],'')) ? $url."&low-price=".$params['lowprice'] : $url.'';
		$url = (strcmp($params['highprice'],'')) ? $url."&high-price=".$params['highprice'] : $url.'';
		$url = (strcmp($params['maxresults'],'')) ? $url."&records-per-page=".$params['maxresults'] : $url.'';
		$url = (strcmp($params['sortby'],'')) ? $url."&sort-by=".$params['sortby'] : $url.'';
		$url = (strcmp($params['sortorder'],'')) ? $url."&sort-order=".$params['sortorder'] : $url.'';
		$url = (strcmp($params['isbn'],'')) ? $url."&isbn=".$params['isbn'] : $url.'';
		$url = (strcmp($params['upc'],'')) ? $url."&upc=".$params['upc'] : $url.'';
		$url = (strcmp($params['manufacturername'],'')) ? $url."&manufacturer-name=".$params['manufacturername'] : $url.'';
		$url = (strcmp($params['manufacturersku'],'')) ? $url."&manufacturer-sku=".$params['manufacturersku'] : $url.'';
		$url = (strcmp($params['advertisersku'],'')) ? $url."&advertiser-sku=".$params['advertisersku'] : $url.'';
		$url = (strcmp($params['lowsaleprice'],'')) ? $url."&low-sale-price=".$params['lowsaleprice'] : $url.'';
		$url = (strcmp($params['highsaleprice'],'')) ? $url."&high-sale-price=".$params['highsaleprice'] : $url.'';
		$url = (strcmp($params['currency'],'')) ? $url."&currency=".$params['currency'] : $url.'';
		
		//Request results from CJ REST API and return results as XML.
		$headers = array( 'Authorization' => $api_key );
		$request = new WP_Http;
		$result = $request->request( $url , array( 'method' => 'GET', 'headers' => $headers, 'sslverify' => false ) );
		if ( is_wp_error($result) ) {
			return $result;
		} else {
		$xml = new SimpleXMLElement($result['body']);
			return $xml;
		}
	}
}

if( class_exists( 'CJ_API_Wordpress' ) ) {
	$cj_api = new CJ_API_Wordpress();
	
	//actions
	add_action('admin_menu', array(&$cj_api,'add_admin_items') );
	add_action( 'admin_init', array(&$cj_api,'register_aioasettings') );
	add_action( 'wp_ajax_cjapi_search', array(&$cj_api, 'ajax_search' ) );
	add_action( 'wp_head', array(&$cj_api, 'add_css' ) );
	add_action( 'init', array(&$cj_api, 'add_tinymce_button'));
	
	//registration hooks
	register_activation_hook( __FILE__, array(&$cj_api, 'activate'));
	register_uninstall_hook( __FILE__, array(&$cj_api, 'uninstall'));
	
	//shortcode
	add_shortcode('cjapi', array(&$cj_api,'cj_product_list') );

}

?>